/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.processor;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.util.config.Property;

/**
 * <summary> A base implementation that can be used for FileProcessors. It provides alias and matcher support,
 * including loading them from configuration file. It serves TextLineCounter and CStyleLineCounter as well as can
 * be genericly used by future implementations. </summary>
 *
 * <configuration> This class can load configuration from a Property. The following properties are supported:
 *
 * <br/> - "alias" string value of the alias for this processor. This property is required and can not be empty.
 *
 * <br/> - "matcher" which specifies a file matcher. It contains a "classname" which specifies the fully qualified
 * class name of a FileMatcher implementation that owns a constructor with a property argument. Optional,
 * AnyFileMatcher is instantiated if not specified. </configuration>
 *
 * <thread-safety> This class is immutable and thread-safe. </thread-safety>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 1.0
 */
public abstract class FileProcessorBase implements FileProcessor {

    /**
     * <summary> This property holds the alias for the file processor. It is required and can not be empty.
     * </summary>
     */
    public static final String ALIAS_PROPERTY_KEY = "alias";

    /**
     * <summary> This property contains sub-properties to specify the attached FileMatcher implementation. When the
     * implementation is instantiated the property itself is passed in as constructor argument. If not specified
     * AnyFileMatcher is used. </summary>
     */
    public static final String FILE_MATCHER_PROPERTY_KEY = "matcher";

    /**
     * <summary> This is a sub-property in "matcher" to specify the fully qualified class name of the FileMatcher
     * implementation that owns a constructor with a Property argument. </summary>
     */
    public static final String CLASSNAME_PROPERTY_KEY = "classname";

    /**
     * <summary> This is the alias for the file processor. It is set in constructor and never changed. </summary>
     */
    private final String alias;

    /**
     * <summary> This is the file matcher for the file processor (and determines if a file can/should be
     * processed). It is set in constructor and never changed. </summary>
     */
    private final FileMatcher matcher;

    /**
     * <summary> Constructor with both alias and matcher. Set them to variables respectively. </summary>
     *
     * @param alias the alias for the processor
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    protected FileProcessorBase(String alias, FileMatcher matcher) {
        checkString(alias, "alias");
        checkNull(matcher, "matcher");

        this.alias = alias;
        this.matcher = matcher;
    }

    /**
     * <summary> This constructor loads configuration from the property. The non-empty alias is required. File
     * matcher is optional (defaults to AnyFileMatcher). Any error will be wrapped in a ConfigurationException.
     * </summary>
     *
     * @param property the property to configure from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    protected FileProcessorBase(Property property) throws ConfigurationException {
        checkNull(property, "property");

        // get the alias form property, empty string is not permitted
        this.alias = property.getValue(ALIAS_PROPERTY_KEY);
        if (alias == null) {
            throw new ConfigurationException("The required property " + ALIAS_PROPERTY_KEY + " is missing");
        }
        if (alias.trim().length() == 0) {
            throw new ConfigurationException("The property " + ALIAS_PROPERTY_KEY
                    + " value must not be an empty string");
        }

        // get the file matcher property and class name
        Property matcherProperty = property.getProperty(FILE_MATCHER_PROPERTY_KEY);
        if (matcherProperty == null) {
            this.matcher = new AnyFileMatcher();
        } else {
            // obtain file matcher class name
            String className = matcherProperty.getValue(CLASSNAME_PROPERTY_KEY);
            if (className == null) {
                throw new ConfigurationException("The required property " + FILE_MATCHER_PROPERTY_KEY + " / "
                        + ALIAS_PROPERTY_KEY + " is missing");
            }

            // create the file matcher by reflection
            try {
                this.matcher = (FileMatcher) Class.forName(className).getConstructor(new Class[] {Property.class})
                        .newInstance(new Object[] {matcherProperty});
            } catch (Exception e) {
                throw new ConfigurationException("Failed to create file matcher", e);
            }
        }
    }

    /**
     * <summary> Get the alias for the file processor. </summary>
     *
     * @return the alias
     */
    public String getAlias() {
        return alias;
    }

    /**
     * <summary> Get the file matcher for the processor. </summary>
     *
     * @return the file matcher
     */
    public FileMatcher getFileMatcher() {
        return matcher;
    }

    /**
     * Checks whether the given Object is not null.
     *
     * @param obj the argument to check
     * @param name the name of the argument
     * @throws NullPointerException if the given Object is null
     */
    public static void checkNull(Object obj, String name) {
        if (obj == null) {
            throw new NullPointerException(name + " should not be null.");
        }
    }

    /**
     * Checks whether the given String is not null and not empty.
     *
     * @param str the String to check
     * @param name the name of the argument
     * @throws NullPointerException if the given string is null
     * @throws IllegalArgumentException if the given string is empty
     */
    public static void checkString(String str, String name) {
        checkNull(str, name);
        if (str.trim().length() == 0) {
            throw new IllegalArgumentException(name + " should not be empty.");
        }
    }

}
