/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Main;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.TypeNameMatcher;
import com.topcoder.util.config.ConfigManager;
import com.topcoder.util.config.ConfigManagerException;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for CStyleLineCounter class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class CStyleLineCounterTests extends TestCase {

    /** An alias string used for tests. */
    private static final String ALIAS = "test alias";

    /** A file matcher used for tests. */
    private static final FileMatcher MATCHER = new AnyFileMatcher();

    /** The CStyleLineCounter instance that is tested. */
    private static final CStyleLineCounter COUNTER = new CStyleLineCounter(ALIAS, MATCHER);

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with null alias. NullPointerException should be thrown.
     */
    public void testCStyleLineCounter1() {
        try {
            new CStyleLineCounter(null, MATCHER);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with null matcher. NullPointerException should be thrown.
     */
    public void testCStyleLineCounter2() {
        try {
            new CStyleLineCounter(ALIAS, null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with empty string matcher. IllegalArgumentException should
     * be thrown.
     */
    public void testCStyleLineCounter3() {
        try {
            new CStyleLineCounter(" ", MATCHER);
            fail("IllegalArgumentException should be thrown");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Creation test of CStyleLineCounter(alias, matcher) with valid arguments.
     */
    public void testCStyleLineCounter4() {
        CStyleLineCounter processor = new CStyleLineCounter(ALIAS, MATCHER);
        assertSame("alias was not properly propagated", processor.getAlias(), ALIAS);
        assertSame("file matcher was not properly propagated", processor.getFileMatcher(), MATCHER);
    }

    /**
     * Creation test of CStyleLineCounter(property) with null property. NullPointerException should be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter5() throws ConfigurationException {
        try {
            new CStyleLineCounter(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for CStyleLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter6a() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
                .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            CStyleLineCounter processor = new CStyleLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof ExtensionMatcher);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for CStyleLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter6b() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance()
                .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor2");
        try {
            CStyleLineCounter processor = new CStyleLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Source File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof TypeNameMatcher);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for CStyleLineCounter with valid configuration property argument.
     *
     * @throws ConfigManagerException to JUnit
     * @throws ConfigurationException to JUnit
     */
    public void testCStyleLineCounter6c() throws ConfigManagerException, ConfigurationException {
        ConfigManager.getInstance().add(Main.NAMESPACE, "conf/FileStatistics_minimal2.xml",
                ConfigManager.CONFIG_XML_FORMAT);
        Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE, "processors.processor1");
        try {
            CStyleLineCounter processor = new CStyleLineCounter(property);
            assertEquals("incorrect alias was configured", processor.getAlias(), "Text File");
            assertTrue("incorrect matcher was configured", processor.getFileMatcher() instanceof AnyFileMatcher);
        } finally {
            ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * Creation test for CStyleLineCounter with invalid configuration namespace arguments. ConfigurationException
     * should be thrown.
     *
     * @throws ConfigManagerException to JUnit
     */
    public void testCStyleLineCounter7() throws ConfigManagerException {
        String[] cases = new String[] {"bad_matcher_classname", "empty_alias", "empty_matcher_extensions",
            "miss_alias", "miss_matcher_classname"};

        for (int i = 0; i < cases.length; i++) {
            String file = "conf/FileStatistics_" + cases[i] + ".xml";
            ConfigManager.getInstance().add(Main.NAMESPACE, file, ConfigManager.CONFIG_XML_FORMAT);
            Property property = ConfigManager.getInstance().getPropertyObject(Main.NAMESPACE,
                    "processors.processor1");

            try {
                new CStyleLineCounter(property);
                fail("ConfigurationException should be thrown. case: " + cases[i]);
            } catch (ConfigurationException e) {
                // ok
            } finally {
                ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);
            }
        }
    }

    /**
     * Tests getAlias() getter method.
     */
    public void testGetAlias() {
        assertSame("alias was not properly propagated", COUNTER.getAlias(), ALIAS);
    }

    /**
     * Tests getFileMachter() getter method.
     */
    public void testGetFlieMatcher() {
        assertSame("file matcher was not properly propagated", COUNTER.getFileMatcher(), MATCHER);
    }

    /**
     * Tests getSupportedMetrics() method. Always singleton list with "line" metric should be returned.
     */
    public void testGetSupportedMetrics() {
        List metrics = COUNTER.getSupportedMetrics();
        assertEquals("incorrect supported metrics list", metrics, Arrays.asList(new Object[] {new Metric(
                CStyleLineCounter.NAME, CStyleLineCounter.DISPLAY_NAME, "some description")}));

        // verify metric name and display name
        Metric metric = (Metric) metrics.get(0);
        assertEquals("incorrect supported metric name", metric.getName(), CStyleLineCounter.NAME);
        assertEquals("incorrect supported metric display name", metric.getDisplayName(),
                CStyleLineCounter.DISPLAY_NAME);
    }

    /**
     * Tests process(File) method with null argument. NullPointerException should be thrown.
     *
     * @throws IOException to JUnit
     */
    public void testProcess1() throws IOException {
        try {
            COUNTER.process(null);
            fail("NullPointerException should be thrown");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with non-exisitng file. IOException should be thrown.
     */
    public void testProcess2() {
        try {
            COUNTER.process(new File("test_files/not_exists"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with a directory file. IOException should be thrown.
     */
    public void testProcess3() {
        try {
            COUNTER.process(new File("test_files/a/"));
            fail("IOException should be thrown");
        } catch (IOException e) {
            // ok
        }
    }

    /**
     * Tests process(File) method with valid files. Number of lines should be returned in stats result.
     *
     * @throws IOException to JUnit
     */
    public void testProcess4() throws IOException {
        checkResult(COUNTER.process(new File("test_files/a/p.txt")), 2);
        checkResult(COUNTER.process(new File("test_files/a/empty.txt")), 0);
        checkResult(COUNTER.process(new File("test_files/b/q.jav")), 83);
        checkResult(COUNTER.process(new File("test_files/c/a.c")), 2);
        checkResult(COUNTER.process(new File("test_files/c/b.c")), 1);
        checkResult(COUNTER.process(new File("test_files/c/c.c")), 4);
    }

    /**
     * Verifies that result contains the given number of lines counted.
     *
     * @param result stats to check
     * @param lines expected number of lines
     */
    private void checkResult(StatsResult result, int lines) {
        assertTrue("there should be only one metric in stats result", result.getMetrics().size() == 1);
        assertEquals("incorrect line metric in stats result", result.getStat(new Metric(CStyleLineCounter.NAME, "-",
                "-")), lines);
    }

}
