/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0
 */
package com.topcoder.file.statistics.processor.linecounter;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import com.topcoder.file.statistics.ArgumentHandler;
import com.topcoder.file.statistics.ArgumentOption;
import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileMatcher;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.processor.FileProcessorBase;
import com.topcoder.util.config.Property;

/**
 * <summary> CStyleLineCounter counts lines for C style source files. This implementation can handle C, C++, C# or
 * Java. It only takes braces and comma into account so it would not be very accurate in most cases. Comments and
 * string literals can be handled. It supports a single metric of "Line Count". </summary>
 *
 * <configuration> Delegates configuration to base class.
 *
 * In addition it has an optional configuration property "support_at_literal" which is a boolean flag that
 * indicates whether verbatim C# literals are processed. It is regarded true when value is either "on", "yes",
 * "true" case-insensitively. This property is optional, default value is false. </configuration>
 *
 * <thread-safety> This class is stateless while its base class is thread-safe (thus it is thread-safe too).
 * </thread-safety>
 *
 * <changes> Since version 2.0b this class adds an option to whether support the at style literal (precisely,
 * verbatim literals from C# spec). This option can come from either configuration file or command line. </changes>
 *
 * @author WishingBone, TCSDEVELOPER
 * @version 2.0b
 */
public class CStyleLineCounter extends FileProcessorBase implements ArgumentHandler {

    /**
     * <summary> The name of the metric used for this processor. </summary>
     */
    public static final String NAME = "Line Count";

    /**
     * <summary> The display name of the metric used for this processor. </summary>
     */
    public static final String DISPLAY_NAME = "line";

    /**
     * <summary> Configuration property key that holds a boolean flag whether to supprt at style literal. This is
     * optional and is evaluated to true if the value is "true"/"yes"/"on" case sensitively. Default value is
     * false. </summary>
     *
     * @since 2.0b
     */
    public static final String SUPPORT_AT_LITERAL_PROPERTY_KEY = "support_at_literal";

    /**
     * <summary> The metric used by this processor. </summary>
     */
    private static final Metric METRIC = new Metric(NAME, DISPLAY_NAME,
        "Number of statements in a source code file.");

    /**
     * <summary> The switch name to configure support at literal by command-line (optional switch).</summary>
     */
    private static final String SUPPORT_AT_LITERAL_SWITCH_NAME = "support_at_literal";

    /**
     * <summary> Flag whether to support at style literal. It's set in constructor or by argument handling and
     * never changes. Default is false</summary>
     *
     * @since 2.0b
     */
    private boolean supportAtLiteral = false;

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * @param alias the alias
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     */
    public CStyleLineCounter(String alias, FileMatcher matcher) {
        super(alias, matcher);
    }

    /**
     * <summary> Propagate the super constructor and set supportAtLiteral flag. </summary>
     *
     * @param alias the alias
     * @param supportAtLiteral the flag to support at style literal
     * @param matcher the file matcher
     *
     * @throws NullPointerException if alias or matcher is null
     * @throws IllegalArgumentException if alias is empty after trimmed
     *
     * @since 2.0
     */
    public CStyleLineCounter(String alias, FileMatcher matcher, boolean supportAtLiteral) {
        super(alias, matcher);
        this.supportAtLiteral = supportAtLiteral;
    }

    /**
     * <summary> Propagate to super constructor. </summary>
     *
     * @param property the property to load config from
     *
     * @throws NullPointerException if property is null
     * @throws ConfigurationException if config error occurs
     */
    public CStyleLineCounter(Property property) throws ConfigurationException {
        super(property);
        String value = property.getValue(SUPPORT_AT_LITERAL_PROPERTY_KEY);
        if (value != null
            && (value.equalsIgnoreCase("yes") || value.equalsIgnoreCase("true") || value.equalsIgnoreCase("on"))) {
            supportAtLiteral = true;
        }
    }

    /**
     * <summary> Return a list contains single Metric composed of the NAME, DISPLAY_NAME and a description.
     * </summary>
     *
     * @return a list of metrics the processor supports
     */
    public List getSupportedMetrics() {
        return Collections.singletonList(METRIC);
    }

    /**
     * <summary> Propose a single ArgumentOption with "support_at_literal" as switch name and required as false.
     * </summary>
     *
     * @return a list of ArgumentOption instances
     * @since 2.0b
     */
    public List proposeOptions() {
        return Collections.singletonList(new ArgumentOption(SUPPORT_AT_LITERAL_SWITCH_NAME,
            "enables C#-style '@' literal support", false));
    }

    /**
     * <summary> Receive command line argument from the command line interface. It does not care about the actual
     * value of the switch (which is generally not provided). It only cares about if the switch is on or off.
     * </summary>
     *
     * @param arguments the arguments from command line
     * @exception NullPointerException if arguments is null
     * @exception IllegalArgumentExcpetion if "support_at_literal" does not correspond to a String
     * @since 2.0b
     */
    public void handleArguments(Map arguments) {
        CheckHelper.checkNull(arguments, "arguments");
        Object value = arguments.get(SUPPORT_AT_LITERAL_SWITCH_NAME);
        if (value == null) {
            supportAtLiteral = false;
        } else if (!(value instanceof String)) {
            throw new IllegalArgumentException("The key '" + SUPPORT_AT_LITERAL_SWITCH_NAME
                + "' does not correspond to a String");
        } else {
            supportAtLiteral = true;
        }
    }

    /**
     * <summary> Count the lines in c style source code file. </summary>
     *
     * <algorithm> 1) Create a text stream. 2) Create a SimpleStateMachine. 3) Process each line with the state
     * machine. 3) Close the stream. 5) Create a StatsResult and place the line count under the specified metric.
     * </algorithm>
     *
     * @return the line count result
     * @param file the file to process
     *
     * @throws NullPointerException if file is null
     * @throws IOException if io operation fails
     */
    public StatsResult process(File file) throws IOException {
        CheckHelper.checkNull(file, "file");

        // create state machine
        SimpleStateMachine state = new SimpleStateMachine(this.supportAtLiteral);

        // read all the file line by line and process with the state machine
        BufferedReader reader = new BufferedReader(new FileReader(file));
        try {
            String line;
            while ((line = reader.readLine()) != null) {
                state.process(line);
            }
        } finally {
            reader.close();
        }

        // return counted number of lines as stats result
        StatsResult result = new StatsResult();
        result.setStat(METRIC, state.getLines());
        return result;
    }

    /**
     * <summary> This is a private static inner class to CStyleLineCounter that implements a state machine that can
     * process the source code. Refer to CS for the state macine description. </summary>
     *
     * <thread-safety> This class is stateful (by name :) and not thread-safe. Each instance only serves a single
     * process. </thread-safety>
     *
     * @author WishingBone, TCSDEVELOPER
     * @version 1.0
     */
    private static class SimpleStateMachine {

        /**
         * <summary> Represents the new line character constant. </summary>
         */
        private static final char NEW_LINE = '\n';

        /**
         * <summary> Represents the state not in comment or literal. </summary>
         */
        private static final int STATE_START = 1;

        /**
         * <summary> Represents the state after a single slash. (Input on slash would enter line comment state;
         * input on star would enter block state.) </summary>
         */
        private static final int STATE_SLASH = 2;

        /**
         * <summary> Represents the state of a star in a block comment. (Input of slash would enter start state,)
         * </summary>
         */
        private static final int STATE_STAR = 3;

        /**
         * <summary> Represents the state of back slash in single quote literal (next character is escaped).
         * </summary>
         */
        private static final int STATE_SQ_BACKSLASH = 4;

        /**
         * <summary> Represents the state of back slash in double quote literal (next character is escaped).
         * </summary>
         */
        private static final int STATE_DQ_BACKSLASH = 5;

        /**
         * <summary> Represents the state in single quote literal. </summary>
         */
        private static final int STATE_SINGLE_QUOTE = 6;

        /**
         * <summary> Represents the state in double quote literal. </summary>
         */
        private static final int STATE_DOUBLE_QUOTE = 7;

        /**
         * <summary> Represents the state in block comment. </summary>
         */
        private static final int STATE_BLOCK_COMMENT = 8;

        /**
         * <summary> Represents the state in line comment. </summary>
         */
        private static final int STATE_LINE_COMMENT = 9;

        /**
         * <summary> Represents the state of before 'at' literal. </summary>
         */
        private static final int STATE_AT = 10;

        /**
         * <summary> Represents the state of in 'at' literal. </summary>
         */
        private static final int STATE_AT_LITERAL = 11;

        /**
         * <summary> Represents any non-special character. </summary>
         */
        private static final int CHAR_OTHER = 0;

        /**
         * <summary> Represents slash character. </summary>
         */
        private static final int CHAR_SLASH = 1;

        /**
         * <summary> Represents star character. </summary>
         */
        private static final int CHAR_STAR = 2;

        /**
         * <summary> Represents backslash character. </summary>
         */
        private static final int CHAR_BACKSLASH = 3;

        /**
         * <summary> Represents single quote character. </summary>
         */
        private static final int CHAR_SINGLE_QUOTE = 4;

        /**
         * <summary> Represents double quote character. </summary>
         */
        private static final int CHAR_DOUBLE_QUOTE = 5;

        /**
         * <summary> Represents line feed character. </summary>
         */
        private static final int CHAR_LINE_FEED = 6;

        /**
         * <summary> Represents the 'at' character. </summary>
         */
        private static final int CHAR_AT = 7;

        /**
         * <summary> An array that maps characters 0..127 into our character codes. </summary>
         */
        private static final int[] CHAR_CODES = prepareCharCodes();

        /**
         * <summary> The length of CHAR_CODES array. </summary>
         */
        private static final int MAX_CHAR = 128;

        /**
         * <summary> An array that maps state and character code into next state. </summary>
         */
        private static final int[][] TRANSITIONS = prepareTransitions();

        /**
         * <summary> The state of this state machine. Reference the consts. </summary>
         */
        private int state = STATE_START;

        /**
         * <summary> The current line count. Initialized to 0 and increments whenever necessary. </summary>
         */
        private long lines = 0;

        /**
         * <summary> A flag whether 'at' literals are enabled. </summary>
         */
        private final boolean enableAtLiteral;

        /**
         * <summary> Default constructor. </summary>
         *
         * @param enableAtLiteral the flag whether 'at' literal processing is enabled.
         */
        public SimpleStateMachine(boolean enableAtLiteral) {
            this.enableAtLiteral = enableAtLiteral;
        }

        /**
         * <summary> Creates and returns an array that maps characters 0..127 into our character codes. </summary>
         *
         * @return array that maps characters 0..127 into our character codes
         */
        private static int[] prepareCharCodes() {
            int[] result = new int[MAX_CHAR];
            // most of characters fall into category "other"
            Arrays.fill(result, CHAR_OTHER);
            // characters relevant for state machine
            char[] chars = new char[] {'/', '*', '\\', '\'', '"', '\n', '@'};
            for (int i = 0; i < chars.length; i++) {
                result[chars[i]] = 1 + i;
            }
            return result;
        }

        /**
         * <summary> Creates and returns an array that maps state and character code into next state. </summary>
         *
         * @return array that maps state and character code into next state
         */
        private static int[][] prepareTransitions() {
            // create the array
            int noOfStates = 1 + STATE_AT_LITERAL;
            int noOfChars = 1 + CHAR_AT;
            int[][] t = new int[noOfStates][noOfChars];

            // most of characters does not change the state
            for (int j = 0; j < noOfStates; j++) {
                Arrays.fill(t[j], j);
            }

            // some states will change to other ones on most cases
            Arrays.fill(t[STATE_SLASH], STATE_START);
            Arrays.fill(t[STATE_STAR], STATE_BLOCK_COMMENT);
            Arrays.fill(t[STATE_SQ_BACKSLASH], STATE_SINGLE_QUOTE);
            Arrays.fill(t[STATE_DQ_BACKSLASH], STATE_DOUBLE_QUOTE);

            // sets the interesting trasnsitions
            t[STATE_START][CHAR_SLASH] = STATE_SLASH;
            t[STATE_START][CHAR_DOUBLE_QUOTE] = STATE_DOUBLE_QUOTE;
            t[STATE_START][CHAR_SINGLE_QUOTE] = STATE_SINGLE_QUOTE;
            t[STATE_SLASH][CHAR_STAR] = STATE_BLOCK_COMMENT;
            t[STATE_SLASH][CHAR_SLASH] = STATE_LINE_COMMENT;
            t[STATE_STAR][CHAR_SLASH] = STATE_START;
            t[STATE_STAR][CHAR_STAR] = STATE_STAR;
            t[STATE_SINGLE_QUOTE][CHAR_BACKSLASH] = STATE_SQ_BACKSLASH;
            t[STATE_SINGLE_QUOTE][CHAR_SINGLE_QUOTE] = STATE_START;
            t[STATE_DOUBLE_QUOTE][CHAR_BACKSLASH] = STATE_DQ_BACKSLASH;
            t[STATE_DOUBLE_QUOTE][CHAR_DOUBLE_QUOTE] = STATE_START;
            t[STATE_LINE_COMMENT][CHAR_LINE_FEED] = STATE_START;
            t[STATE_BLOCK_COMMENT][CHAR_STAR] = STATE_STAR;

            // changes since 2.0b
            // state AT works similarly to START
            Arrays.fill(t[STATE_AT], STATE_START);
            t[STATE_AT][CHAR_SLASH] = STATE_SLASH;
            t[STATE_AT][CHAR_DOUBLE_QUOTE] = STATE_DOUBLE_QUOTE;
            t[STATE_AT][CHAR_SINGLE_QUOTE] = STATE_SINGLE_QUOTE;
            // except on dobule quote, when we enter verbatim ('at') literal
            t[STATE_AT][CHAR_DOUBLE_QUOTE] = STATE_AT_LITERAL;
            // a quote is probably an exit from state literal
            t[STATE_AT_LITERAL][CHAR_DOUBLE_QUOTE] = STATE_AT;
            // the START -> AT transition is enabled dynamically

            // return the created transitions array
            return t;
        }

        /**
         * <summary> Process a line. It processes each character as well as a line feed. </summary>
         *
         * @param line the line to process
         *
         * @throws NullPointerException if line is null
         */
        public void process(String line) {
            CheckHelper.checkNull(line, "line");
            for (int i = 0; i < line.length(); i++) {
                process(line.charAt(i));
            }
            process(NEW_LINE);
        }

        /**
         * <summary> Process a character from input. Refer to CS for algorithm. </summary>
         *
         * @param ch the character to process
         */
        private void process(char ch) {
            if (state == STATE_START && (ch == ';' || ch == '{')) {
                lines++;
            }

            // since 2.0b; to enable the transition edge START -> AT
            if (enableAtLiteral && state == STATE_START && ch == '@') {
                state = STATE_AT;
                return;
            }

            // change to the next state
            state = TRANSITIONS[state][CHAR_CODES[ch >= MAX_CHAR ? MAX_CHAR - 1 : ch]];
        }

        /**
         * <summary> Get the line count so far. Simply return the variable. </summary>
         *
         * @return the line count
         */
        public long getLines() {
            return lines;
        }
    }

}
