/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics;

import java.io.File;
import java.util.List;
import java.util.regex.Pattern;

import junit.framework.TestCase;

import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.matcher.BinaryFileMatcher;
import com.topcoder.file.statistics.matcher.ExtensionMatcher;
import com.topcoder.file.statistics.matcher.MimeMatcher;
import com.topcoder.file.statistics.matcher.RegexMatcher;
import com.topcoder.file.statistics.matcher.TextFileMatcher;
import com.topcoder.file.statistics.matcher.TypeNameMatcher;
import com.topcoder.file.statistics.processor.linecounter.CStyleLineCounter;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.file.statistics.reporting.BasicStatsReporting;
import com.topcoder.file.statistics.reporting.XmlStatsReporting;
import com.topcoder.util.config.ConfigManager;

/**
 * <p>Component demonstration for File Statistics.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class Demo extends TestCase {

    /**
     * Sets up the test case, load the default namespace.
     *
     * @throws Exception to JUnit
     */
    protected void setUp() throws Exception {
        ConfigManager.getInstance()
            .add(Main.NAMESPACE, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);
    }

    /**
     * Disposes the test case, removes configuration namespace.
     *
     * @throws Exception to JUnit
     */
    protected void tearDown() throws Exception {
        ConfigManager cm = ConfigManager.getInstance();
        if (cm.existsNamespace(Main.NAMESPACE)) {
            cm.removeNamespace(Main.NAMESPACE);
        }
    }

    /**
     * <p>Demonstrates the general usage of File Statistics. Based on the 4.3 Demo section of Component
     * Specification.</p>
     *
     * @throws Exception to JUnit
     */
    public void testUsageDemo() throws Exception {
        // 4.3.1 Configure File Statistics

        // create a file statistics instance
        ExtendedFileStatistics statistics = new ExtendedFileStatistics();

        // create file matchers
        FileMatcher matcher1 = new AnyFileMatcher();
        FileMatcher matcher2 = new ExtensionMatcher("txt");
        FileMatcher matcher3 = new RegexMatcher(Pattern.compile("pattern"));
        FileMatcher matcher4 = new TextFileMatcher();
        FileMatcher matcher5 = new BinaryFileMatcher();
        FileMatcher matcher6 = new TypeNameMatcher("Java");
        FileMatcher matcher7 = new MimeMatcher("text/plain");

        // create file processors
        FileProcessor processor1 = new TextLineCounter("Text File", matcher1);
        FileProcessor processor2 = new CStyleLineCounter("Source File", matcher6);

        // create stats reporting implementation
        StatsReporting reporting1 = new BasicStatsReporting();
        StatsReporting reporting2 = new XmlStatsReporting(new File("test_files/stylesheet/xml_to_html.xsl"));

        // add file processors
        statistics.addFileProcessor(processor1);
        statistics.addFileProcessor(processor2);

        // manipulate reporting
        StatsReporting original = statistics.getReporting();
        statistics.setReporting(reporting2);

        // manipulate match all flag
        if (!statistics.isMatchAll()) {
            statistics.setMatchAll(true);
        }

        // 4.3.2 Process File
        // process single file
        statistics.processFile(new File("test_files/a/p.txt"));
        statistics.processFile(new File("test_files/b/q.jav"));

        // process directory
        statistics.processDirectory(new File("test_files/a"));
        statistics.processDirectory(new File("test_files"), true);

        // 4.3.3 Generate Report
        // obtain report
        String report = statistics.getReporting().generateReport();

        // output report
        statistics.printReport();
        statistics.printReport(new File("test_files/output.html"));

        // reset reporting
        statistics.getReporting().reset(new File("test_files"));

        // manipulate file processors
        FileProcessor removed = statistics.removeFileProcessor("Source File");
        FileProcessor query = statistics.getFileProcessor("Text File");
        List processors = statistics.getAllFileProcessors();
        statistics.clearAllFileProcessors();

    }

    /**
     * <p>Demonstrates the command line usage of File Statistics. Based on the 4.3.4 Demo section of Component
     * Specification. Will call the Main.main(args) method directly, however, it is to test and demonstrate the
     * command line usage.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo1() throws Exception {

        // this test requires the Main.NAMESPACE not to be loaded
        ConfigManager.getInstance().removeNamespace(Main.NAMESPACE);

        // specify configuration file
        // -c conf/FileStatistics.xml test_files/a.txt test_files/b.jav
        String[] args = new String[] {"-c", "conf/FileStatistics.xml", "test_files/a.txt", "test_files/b.jav"};
        Main.main(args);
    }

    /**
     * <p>Demonstrates the command line usage of File Statistics. Based on the 4.3.4 Demo section of Component
     * Specification. Will call the Main.main(args) method directly.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo2() throws Exception {
        // specify namespace
        // -n com.topcoder.file.statistics test_files/a.txt test_files/b.jav
        String namespace = "com.topcoder.file.statistics";

        // the namespace must be preloaded
        ConfigManager.getInstance().add(namespace, "conf/FileStatistics.xml", ConfigManager.CONFIG_XML_FORMAT);

        try {
            Main.main(new String[] {"-n", "com.topcoder.file.statistics", "test_files/a.txt", "test_files/b.jav"});

        } finally {
            ConfigManager.getInstance().removeNamespace(namespace);
        }
    }

    /**
     * <p>Demonstrates the command line usage of File Statistics. Based on the 4.3.4 Demo section of Component
     * Specification. Will call the Main.main(args) method directly.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo3() throws Exception {
        // specify base directory
        // -b test_files test_files/a.txt test_files/b.jav

        Main.main(new String[] {"-b", "test_files", "a.txt", "b.jav"});
    }

    /**
     * <p>Demonstrates the command line usage of File Statistics. Based on the 4.3.4 Demo section of Component
     * Specification. Will call the Main.main(args) method directly.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo4() throws Exception {
        // recursively process sub-directory
        // -r test_files
        Main.main(new String[] {"-r", "test_files"});
    }

    /**
     * <p>Demonstrates the command line usage of File Statistics. Based on the 4.3.4 Demo section of Component
     * Specification. Will call the Main.main(args) method directly.</p>
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo5() throws Exception {
        // specify output file
        // -o test_files/output.html test_files/a.txt test_files/b.java
        Main.main(new String[] {"-o", "test_files/output/Demo5.html", "test_files/a.txt", "test_files/b.jav"});
    }

    /**
     * <p>Demonstrates the command line usage of File Statistics. A stylesheet is specified by command line switch</p>
     *
     * @since 2.0b
     *
     * @throws Exception to JUnit
     */
    public void testCommandLineDemo6() throws Exception {
        // will use specified stylesheet
        Main.main(new String[] {"-o", "test_files/output/Demo6.txt", "-stylesheet", "test_files/stylesheet/2b.xsl",
            "test_files/a.txt", "test_files/b.jav"});
    }

    /**
     * <p>Prints a demonstration of command line usage of File Statistics. </p>
     *
     * @since 2.0b
     *
     * @throws Exception to JUnit
     */
    public void testCommandLinePrintUsageDemo() throws Exception {
        // for no arguments, should just print a usage
        Main.main(new String[] {});
    }

}
