/*
 * Copyright (C) 2005 TopCoder Inc., All Rights Reserved.
 *
 * TCS File Statistics 1.0 (Unit Test)
 */
package com.topcoder.file.statistics.reporting;

import java.io.File;

import junit.framework.TestCase;

import com.topcoder.file.statistics.ConfigurationException;
import com.topcoder.file.statistics.FileProcessor;
import com.topcoder.file.statistics.Metric;
import com.topcoder.file.statistics.StatsReportingException;
import com.topcoder.file.statistics.StatsResult;
import com.topcoder.file.statistics.TestHelper;
import com.topcoder.file.statistics.matcher.AnyFileMatcher;
import com.topcoder.file.statistics.processor.linecounter.TextLineCounter;
import com.topcoder.util.config.Property;

/**
 * <p>Unit test case for BasicStatsReporting class.</p>
 *
 * @author TCSDEVELOPER
 * @version 1.0
 */
public class BasicStatsReportingTests extends TestCase {

    /** A file processor used for tests. */
    private static final FileProcessor PROCESSOR = new TextLineCounter("test alias", new AnyFileMatcher());

    /** A metric used in tests. */
    private static final Metric METRIC = new Metric("name", "lines", "test description");

    /** A stat value used in tests. */
    private static final int STAT_VALUE = 22;

    /** A file name used for tests. */
    private static final String FILE_NAME = new File("test_files/a/p.txt").getPath();

    /** A dir name used for tests. */
    private static final String DIR_NAME = new File("test_files/a").getPath();

    /** The instance of tested class. */
    private BasicStatsReporting reporting;

    /**
     * Sets up the test. Initializes tested instance.
     *
     * @throws StatsReportingException to JUnit
     */
    protected void setUp() throws StatsReportingException {
        reporting = new BasicStatsReporting();

        StatsResult result = new StatsResult();
        result.setStat(METRIC, STAT_VALUE);
        reporting.aggregate(PROCESSOR, new File(FILE_NAME), result);
    }

    /**
     * Creation test for BasicStatsReporting default constructor.
     */
    public void testBasicStatsReporting1() {
        BasicStatsReporting newReporting = new BasicStatsReporting();
        assertNull("new reporting should have no base directory set", newReporting.getBaseDirectory());
        assertTrue("new reporting should have empty stats", newReporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getDirectoryStats().getAllKeys()
                .isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Creation test for BasicStatsReporting(Property) constructor with null argument. NullPointerException should
     * be thrown.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testBasicStatsReporting2() throws ConfigurationException {
        try {
            new BasicStatsReporting(null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }
    }

    /**
     * Creation test for BasicStatsReporting(Property) constructor with valid argument.
     *
     * @throws ConfigurationException to JUnit
     */
    public void testBasicStatsReporting3() throws ConfigurationException {
        BasicStatsReporting newReporting = new BasicStatsReporting(new Property("test name"));

        assertNull("new reporting should have no base directory set", newReporting.getBaseDirectory());
        assertTrue("new reporting should have empty stats", newReporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getDirectoryStats().getAllKeys()
                .isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("new reporting should have empty stats", newReporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Tests getAliasStats() getter method.
     */
    public void testGetAliasStats() {
        StatsCollection aliasStats = reporting.getAliasStats();
        assertEquals("incorrect alias stats", aliasStats.get(PROCESSOR.getAlias()).getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getFileStats() getter method.
     */
    public void testGetFileStats() {
        StatsCollection fileStats = reporting.getFileStats();
        assertEquals("incorrect file stats", fileStats.get(FILE_NAME).getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getDirectoryStats() getter method.
     */
    public void testGetDirectoryStats() {
        StatsCollection dirStats = reporting.getDirectoryStats();
        assertEquals("incorrect directory stats", dirStats.get(DIR_NAME).getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getOverallStats() getter method.
     */
    public void testGetOverallStats() {
        StatsResult overallStats = reporting.getOverallStats();
        assertEquals("incorrect overall stats", overallStats.getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests getBaseDirectory() getter method.
     */
    public void testGetBaseDirectory() {
        File baseDir = reporting.getBaseDirectory();
        assertNull("incorrect base directory", baseDir);
    }

    /**
     * Tests reset(File) method with valid directory. The stats should be cleared, and base directory should be
     * set.
     */
    public void testReset1() {
        File dir = new File(DIR_NAME);
        reporting.reset(dir);
        assertSame("incorrect base directory", dir, reporting.getBaseDirectory());

        assertTrue("reporting should have empty stats", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Tests reset(File) method with null directory. The stats should be cleared, and base directory should be set
     * to null.
     */
    public void testReset2() {
        reporting.reset(null);
        assertNull("incorrect base directory", reporting.getBaseDirectory());

        assertTrue("reporting should have empty stats", reporting.getFileStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getAliasStats().getAllKeys().isEmpty());
        assertTrue("reporting should have empty stats", reporting.getOverallStats().getMetrics().isEmpty());
    }

    /**
     * Tests reset(File) method with invalid argument. IllegalArgumentException should be thrown.
     */
    public void testReset3() {
        // not a directory
        File file = new File(FILE_NAME);
        try {
            reporting.reset(file);
            fail("IllegalArgumentException");
        } catch (IllegalArgumentException e) {
            // ok
        }
    }

    /**
     * Tests aggregate method with valid arguments. Stats should be updated appropriately.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testAggregate1() throws StatsReportingException {
        reporting.reset(null);
        StatsResult result = new StatsResult();
        result.setStat(METRIC, STAT_VALUE);

        reporting.aggregate(PROCESSOR, new File(FILE_NAME), result);

        assertEquals("incorrect alias stats", reporting.getAliasStats().get(PROCESSOR.getAlias()).getStat(METRIC),
                STAT_VALUE);
        assertEquals("incorrect file stats", reporting.getFileStats().get(FILE_NAME).getStat(METRIC), STAT_VALUE);
        assertEquals("incorrect directory stats", reporting.getDirectoryStats().get(DIR_NAME).getStat(METRIC),
                STAT_VALUE);
        assertEquals("incorrect overall stats", reporting.getOverallStats().getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests aggregate method with some null argument. NullPointerException should be thrown.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testAggregate2() throws StatsReportingException {
        try {
            reporting.aggregate(null, new File(FILE_NAME), new StatsResult());
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }

        try {
            reporting.aggregate(PROCESSOR, null, new StatsResult());
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }

        try {
            reporting.aggregate(PROCESSOR, new File(FILE_NAME), null);
            fail("NullPointerException should be thrown.");
        } catch (NullPointerException e) {
            // ok
        }

    }

    /**
     * Tests aggregate method with valid arguments with baseDir set. Stats should be updated appropriately, and
     * file name should be stripped off the base dir part.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testAggregate3() throws StatsReportingException {
        reporting.reset(new File(DIR_NAME));
        StatsResult result = new StatsResult();
        result.setStat(METRIC, STAT_VALUE);

        reporting.aggregate(PROCESSOR, new File(FILE_NAME), result);

        assertEquals("incorrect alias stats", reporting.getAliasStats().get(PROCESSOR.getAlias()).getStat(METRIC),
                STAT_VALUE);
        assertEquals("incorrect file stats", reporting.getFileStats().get(new File(FILE_NAME).getName()).getStat(
                METRIC), STAT_VALUE);
        assertTrue("incorrect directory stats", reporting.getDirectoryStats().getAllKeys().isEmpty());
        assertEquals("incorrect overall stats", reporting.getOverallStats().getStat(METRIC), STAT_VALUE);
    }

    /**
     * Tests aggregate method with file name outside base directory. StatsReportingException should be thrown.
     */
    public void testAggregate4() {
        reporting.reset(new File(DIR_NAME));
        try {
            reporting.aggregate(PROCESSOR, new File("outside/base/dir"), new StatsResult());
            fail("StatsReportingException should be thrown.");
        } catch (StatsReportingException e) {
            // ok
        }
    }

    /**
     * Tests generateReport() method.
     *
     * @throws StatsReportingException to JUnit
     */
    public void testGenerateReport() throws StatsReportingException {
        // prepare some interesting results
        StatsResult stats = new StatsResult();
        stats.setStat(METRIC, 6);
        stats.setStat(new Metric("metric 2", "bugs", "some description..."), 13);
        reporting.aggregate(PROCESSOR, new File("test_files/b/q.java"), stats);

        String report = reporting.generateReport();

        // reggresion test
        assertEquals("incorrect report", report, TestHelper.readFile("test_files/basic_stats_report.txt"));
    }

}
